    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
%
    n=34;
    a=sprand(n,n,0.2)+10*eye(n); % define sparse random matrix
    asave=a;
    full(a);
%
%  ILU0 
%
    a=asave;
    tau=0.0;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... Right-looking ILU(0), tau =',tau)
    [l,u]=ilu0(a);
    fprintf('%s %1.8g \n','     ILU(0): residual norm(a-l*u) = ',norm(full(asave-l*u)))
    x=ones(n,1);b=asave*x;m1=l*u;m2=eye(n);rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d \n','     GMRES/ILU(0): flag, rtol, its = ',flag,rtol,iter)
%
%  ILU(tau) Crout algorithm
%
    a=asave;
    tau=0.0;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... Crout ILU(tau), tau =',tau)
    [l,u]=ilu_crout(a,tau);
    fprintf('%s %1.8g \n','     Crout ILU: residual norm(a-l*u) = ',norm(full(asave-l*u)))
    x=ones(n,1);b=asave*x;m1=l*u;m2=eye(n);rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d \n','     GMRES/Crout ILU: flag, rtol, its = ',flag,rtol,iter)
%
%  ILU by rows
%
    a=asave;
    
    tau=0.01;
    tau=0;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... ILU by rows, tau =',tau)
    [l,u]=ilu_rows(a,tau);
    fprintf('%s %1.8g \n','     ILU by rows: residual norm(a-l*u) = ',norm(full(asave-l*u)))
    x=ones(n,1);b=asave*x;m1=l*u;m2=eye(n);rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d \n','     GMRES/ILU by rows: flag, rtol, its = ',flag,rtol,iter)
%
%   ILU by levels
%
    a=asave;
    nlev=0;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... ILU by levels (standard search for the target pattern), number of levels =',nlev)
    [l,u]=ilulevel2(a,nlev);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))
    full(l+u); 
    x=ones(n,1);b=asave*x;m1=l*u;m2=eye(n);rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d \n','     GMRES/ILU by levels: flag, rtol, its = ',flag,rtol,iter)
%
%   ILU by levels with fast search for the target sparsity pattern
%
    a=asave;
    nlev=3;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... ILU by levels (fast search for the target pattern), number of levels =',nlev)
    pattern=get_ilu_pattern(a,nlev);
    [l,u]=ilu_pattern(a,pattern);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))
    full(l+u);
    norm(l*u -a);
    %
    %   MILU(0) (Chan, van der Vorst, 1994)
    %
    a=asave;
    fprintf('\n')
    fprintf('%s \n',' ... MILU(0) (Chan, van der Vorst)')
    [l,u]=milu0(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))

    fullproduct=full(l*u);
    rs=row_sums_a(a);
    rslu=row_sums_a(fullproduct);
    fprintf('%s %1.8g \n','     MILU(0): difference in row counts A, M = ',norm(rs-rslu))

    a=asave;
    fprintf('\n')
    fprintf('%s \n',' ... MILU(0) (just a variation)')
    [l,u]=milu0a(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))
    fullproduct=full(l*u);
    rs=row_sums_a(a);
    rslu=row_sums_a(fullproduct);
    fprintf('%s %1.8g \n','     MILU(0): difference in row counts A, M = ',norm(rs-rslu))
    %
    %  MILU(pattern) 
    %
    a=asave;
    fprintf('\n')
    fprintf('%s \n',' ... simple MILU(pattern) ')
    [n n]=size(a);
    b=sprand(n,n,0.3)+10*eye(n); % define sparse random matrix (for MILU pattern)
%   b=ones(n,n);
    [l,u]=milu_simple(a,b);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))

    fullproduct=full(l*u);
    rs=row_sums_a(a);
    rslu=row_sums_a(fullproduct);
    fprintf('%s %1.8g \n','     simple MILU: difference in row counts A, M = ',norm(rs-rslu))
    %
    %  MILU(pattern) 
    %
    a=asave;
    fprintf('\n')
    fprintf('%s \n',' ... MILU(pattern) ')
    [n n]=size(a);
    b=sprand(n,n,0.3)+10*eye(n); % define sparse random matrix (for MILU pattern)
%   b=ones(n,n);
    [l,u]=milu(a,b);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))

    fullproduct=full(l*u);
    rs=row_sums_a(a);
    rslu=row_sums_a(fullproduct);
    fprintf('%s %1.8g \n','     MILU: difference in row counts A, M = ',norm(rs-rslu))
%
%   ILU(tau) (right-looking) (direct setting of L and U), Ajiz-Jennings
%
    a=asave;
    tau=0.0001; tau=0.1;
    pattern_type=1; % a
    pattern_type=10; % full pattern

    fprintf('\n')
    fprintf('%s  %1.8g %d \n',' ... right-looking ILU(tau) - version Ajiz-Jennings, tau, pattern_type =',tau,pattern_type)
    pattern=get_pattern(a,pattern_type);
    omega=1.0;
    [l,u]=ilu_kij_aj_pattern(a,tau,omega,pattern);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))
    x=ones(n,1);b=asave*x;m1=l*u;m2=eye(n);rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d \n','     GMRES/ILU Ajiz-Jennings: flag, rtol, its = ',flag,rtol,iter)
%
%   IC memory limited
%
    fprintf('\n')
    a=asave'*asave;
    tau=0.0;
    lsize=10; rsize=10;
    fprintf('%s %1.8g %d %d \n',' ... IC memory-limited, tau, lsize, rsize = ',tau,lsize,rsize)
    [l,d]=icl_ad_me_ti(a,tau,lsize,rsize);
    fprintf('%s %1.8g \n','     residual norm(a-l*d*l^T) = ',norm(full(a-l*d*l')))
%
%   ILU (fixed-point) new version (direct setting of L and U), Ajiz-Jennings
%
    a=asave;
    nsweep=1;
    fprintf('\n')
    fprintf('%s  %d \n',' ... ILU fixed point, nsweeps =',nsweep)
    [l,u]=ilu_fixed_point(a,a,a,nsweep);
    fprintf('%s %1.8g \n','     residual norm(a-l*u) = ',norm(full(asave-l*u)))
    x=ones(n,1);b=asave*x;m1=l*u;m2=eye(n);rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d \n','     GMRES/ILU fixed point: flag, rtol, its = ',flag,rtol,iter)
    %
    % incomplete Cholesky sparse CSC (tau)
    %
    a=asave;
    a=a'*a; % must be SPD here
%    a=poisson(2);
    a=full(a);
    [n n]=size(a);
    fprintf('\n')
    [ia,ja,aa]=a2csr(a);
    [ia,ja,aa]=a2csr_sparse(a);             % transform the matrix a into CSR
    tau=0.0;
    fprintf('\n')
    fprintf('%s %1.8g \n',' ... incomplete Cholesky sparse CSC (tau), tau = ',tau)
    [ib,jb,ab]=unsymtr7(n,n,ia,ja,aa); % unsymmetrize
    [il,jl,al]=icl_CSC(n,ib,jb,ab,tau);
    %
    % compute L^T*L
    %
    [ilt,jlt,alt]=trancs(n,n,il,jl,al); % il,jl,al is the transposed factor by rows
    [ic,jc,ac]=matmat_rr(n,n,ilt,jlt,alt,n,n,il,jl,al); % multiplies l*l'
    [ic,jc,ac]=mmadd(n,n,1,ia,ja,aa,-1,ic,jc,ac); % add matrices
    %
    csra=struct;csra.nrows=n;csra.ncols=n;csra.ia=ia;csra.ja=ja;csra.aa=aa; 
    csrp=struct;csrp.nrows=n;csrp.ncols=n;csrp.ia=il;csrp.ja=jl;csrp.aa=al;
    csrc=struct;csrc.nrows=n;csrc.ncols=n;csrc.ia=ic;csrc.ja=jc;csrc.aa=ac;
    nrm=normf_CSR(csrc);
    nrm=normi_CSR(csrc);
    x=ones(n,1); b=mv_CSR(csra,x);
    u=fsolve_CSC(csrp,b);
    x=bsolve_CSC(csrp,u);
    fprintf('%s %1.8g \n','     residual norm(a-l*l^T) = ', nrm)
    [ll]=[chol(a)]';
    b=ones(n,1);m1=ll;m2=m1';rtol=1.0d-8;maxit=1000;prtyp=31;
    [x,relres,backerr,iter,info]=cg_CSR(csra,b,rtol,maxit,csrp,prtyp);
    fprintf('%s %d %1.8g %d\n','     PCG: rtol, its = ',flag,rtol,iter)
    %
    % incomplete Cholesky experimental (S1,S2) (tau)
    %
    a=poisson(33);
    a=asave'*asave;
    a=full(a);
    [n n]=size(a);
    tau=0.01;
    fprintf('\n')
    fprintf('%s %1.8g \n',' ... incomplete Cholesky(tau) experimental, tau = ',tau)
    %  mode: = 0: standard (S1)
    %        = 1: up_looking-based (S2)
    l0=icl2(a,tau,0); nzl0=nz_a(l0);
    l1=icl2(a,tau,1); nzl1=nz_a(l1);
    fprintf('%s %1.8g \n','     residual norm(a-l*l^T) S1= ',norm(full(a-l0*l0')))
    b=ones(n,1);m1=l0;m2=l0';rtol=1.0d-8;maxit=1000;
    [x,flag,relres,iter]=pcg(a,b,rtol,maxit,m1,m2);
    fprintf('%s %d %1.8g %d %d \n','     PCG: flag, rtol, its, nzl0: ',flag,rtol,iter,nzl0)
    fprintf('%s %1.8g \n','     residual norm(a-l*l^T) S2= ',norm(full(a-l1*l1')))
    b=ones(n,1);m1=l1;m2=l1';rtol=1.0d-8;maxit=1000;
    [x,flag,relres,iter]=pcg(a,b,rtol,maxit,m1,m2);
    fprintf('%s %d %1.8g %d %d \n','     PCG: flag, rtol, its, nzl1: ',flag,rtol,iter,nzl1)
    %
    %  incomplete LDLT dense (tau)
    %
    a=asave;
    a=a'*a; 
    a=poisson(13);
    [n n]=size(a);
    tau=0.0;                                                               
    fprintf('\n')
    fprintf('%s %1.8g \n',' ... incomplete LDLT(tau), tau = ',tau)
    [l,d]=ildlt(a,tau);
    fprintf('%s %1.8g \n','     residual norm(a-l*d*l^T) = ',norm(full(a-l*d*l')))
    x=ones(n,1);b=a*x;m1=l*sqrt(d);m2=m1';rtol=1.0d-8;maxit=1000;x0=[];
    [x,flag,relres,iter]=pcg(a,b,rtol,maxit,m1,m2,x0);
    fprintf('%s %d %1.8g %d\n','     PCG: flag, rtol, its = ',flag,rtol,iter)
    %
    %  incomplete Cholesky dense (tau)
    %
    fprintf('\n')
    a=asave'*asave;
    asave=a;
    [n n]=size(a);
    tau=0.01;
    fprintf('%s %1.8g \n',' ... incomplete Cholesky(tau), tau = ',tau)
    [ll]=icl(a,tau);
    fprintf('%s %1.8g \n','     residual norm(a-l*l^T) = ',norm(full(a-ll*ll')))
    b=ones(n,1);m1=ll;m2=m1';rtol=1.0d-8;maxit=1000;
    [x,flag,relres,iter]=pcg(a,b,rtol,maxit,m1,m2);
    fprintf('%s %d %1.8g %d\n','     PCG: rtol, its = ',flag,rtol,iter)
    %
    % IC(tau)
    %
    fprintf('\n')
    a=asave;
    tau=0.00001;
    fprintf('%s %1.8g \n',' ... IC(tau) (a different one), tau = ',tau)
    l=ictau(a,0.00001);
    fprintf('%s %1.8g \n','     residual norm(a-l*l^T) = ',norm(full(asave-l*l')))
    %
    % IC(tau) + ajiz, jennings
    %
    a=asave;
    tau=0.00001;
    fprintf('\n')
    fprintf('%s %1.8g \n',' ... IC(tau) + Ajiz-Jennings , tau = ',tau)
    l=ictau_aj(a,tau);
    fprintf('%s %1.8g \n','     residual norm(a-l*l^T) = ',norm(full(asave-l*l')))
    %
    % IC(0)  - right-looking in LDL' form
    %
    a=asave;
    fprintf('\n')
    fprintf('%s \n',' ... right-looking IC(0) in LDL^T form')
    [l,d]=ic0(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*d*l^T) = ',norm(full(asave-l*d*l')))
    %
    % IC(0)  - left-looking Cholesky
    %
    a=asave;
    fprintf('\n')
    fprintf('%s \n',' ... another right-looking Cholesky IC(0)')
    [l nonpos]=ic0a(a);
    if (nonpos==0) 
      fprintf('%s %1.8g \n','     residual norm(a-l*lT) = ',norm(full(asave-l*l')))
    else
      fprintf('%s \n','     factorization breakdown')
    end
    %                
    % ic by levels
    %
    a=asave;
    ilev=1;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... IC by levels, number of levels =',ilev)
    l=iclevel(a,ilev);
    fprintf('%s %1.8g \n','     residual norm(a-l*l^T) = ',norm(full(asave-l*l')))
    %  spy(l+l')
    %
    % MIC(0) (Chan, van der Vorst, 1994)
    %
    a=asave;
    fprintf('\n')
    fprintf('%s \n',' ... MIC(0) in LDL^T form')
    [l,d]=mic0(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*d*l^T) = ',norm(full(asave-l*d*l')))

    %
    % IC(tau) with inverse-based dropping
    %
    a=asave;
    tau=0.0;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... right-looking IC(tau) with inverse-based dropping, tau =',tau)
    [l,d]=iclid(a,tau);
    fprintf('%s %1.8g \n','     residual norm(a-l*d*l^T) = ',norm(full(asave-l*d*l')))
    %
    % column Cholesky (just to show that the Theorem is OK)
    %
    a=asave;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... column Cholesky')
    [ll]=ch_columns(a); % probably incorrect
    fprintf('%s %1.8g \n','     column Cholesky: residual norm(a-l*lT) = ',norm(full(asave-ll*ll')))
    %
    % column LDLT (just to show that the Theorem is OK)
    %
    a=asave;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... column LDLT')
    [l,d]=ldlt_columns(a);
    fprintf('%s %1.8g \n','     column LDLT: residual norm(a-l*d*lT) = ',norm(full(asave-l*d*l')))
    %
    % example by Kershaw showing negative diagonal entry
    %
    tau=0.0;
    tol=1.0d-15;	
    fprintf('\n')
    fprintf('%s  \n',' ... example by Kershaw showing breakdown')
    [l,d,posdef]=kershaw(tol);
    if(posdef == 0) 
      fprintf('%s \n','     breakdown in IC(0)') 
    end
    %
    % find red-black reordering
    %
    a=asave;
    a=poisson(3);
    [m n]=size(a);
    fprintf('\n')
    fprintf('%s  \n',' ... find red-black ordering')
    flag=zeros(n,1);
    istart=1;
    [ia,ja,aa]=a2csr(a);
    [bipartite] = is_bipartite(n,ia,ja,aa);
    if(bipartite == 0) % bipartite graph
      fprintf('%s \n','     the matrix undirected model is bipartite') 
      [p]=rb(a);
      fprintf('\n')
      fprintf('%s  \n',' ... Red-black ordering found')
      a=a(p,p);
      full(a);
    end
    %
    % transform matrix into block tridiagonal form
    % UNFINISHED
    if(never) 
      a=asave;
      [ia ja aa]=a2csr(a);
      [n n]=size(a);
      fprintf('\n')
      fprintf('%s  \n',' ... transform into block tridiagonal form')
      [blsize,ib,jb,ab,pb,isbtriag]=find_btridiag(n,ia,ja,aa);
      a=btriag2a(blsize,n,ib,jb,ab)
      norm(asave-a)
      %
      % ICB - IC with uniform block size
      %
      nb=4;  
      [l,d]=icb(a,nb,0.01);
      fprintf('%s %1.8g \n','residual a-l*d*lT for ICB(tau) (block IC with uniform blocks) = ',norm(full(a-l*d*l')))
    end

