    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
    %
    % chapter_1_introduction
    %
    fprintf('\n')
    fprintf('%s \n',' ... getting initial matrix')
    n=8;
    a=sprand(n,n,0.5)+10*eye(n); % define sparse random matrix
    a=poisson(3);
    full(a)
    figure(1);clf;
    spy(a)
    %
    % example permutation
    %
    fprintf('\n')
    fprintf('%s \n',' ... getting permutation')
    ind=1;
    for i=1:2:n
      p(i)=ind;
      ind=ind+1;
    end
    for i=2:2:n
      p(i)=ind;
      ind=ind+1;
    end
    %
    % see matrix pattern
    %
    figure(2);clf;
    p=p(1:n); 
    a=a(p,p)
    spy(a)
    %
    % another matrix
    %
    fprintf('\n')
    fprintf('%s \n',' ... getting another matrix')
    fprintf('%s \n',' ... showing factor patterns (original, permuted by AMD')
    a = mmread('orsirr2.mtx');
    a=a'*a;
    [m n]=size(a);
    a=a+a'+n*eye(n);
    chol(a);
    figure(3);clf;
    l=chol(a); l=l+l';
    spy(l)
    p=symamd(l);
    l=chol(a(p,p)); l=l+l';
    figure(4);clf;
    spy(l)
    %
    % convert matrix into CSR format
    %
    n=8;
    a=sprand(n,n,0.5)+10*eye(n); % define sparse random matrix
    [m n]=size(a);
    full(a)
    %
    fprintf('\n')
    fprintf('%s \n',' ... converting a matrix into CSR')
    [ia ja aa]=a2csr(a);
    ia=ia'; ja=ja';  aa=aa'; % technical stuff only
    %
    %  transpose the matrix in CSR
    %  
    fprintf('\n')
    fprintf('%s \n',' ... transposition in CSR')
    [iat,jat,aat]=trancs(m,n,ia,ja,aa);
    at=csr2a(m,iat,jat,aat); % just for fun

    %
    % transpose the matrix back in CSR 
    %
    [ic,jc,ac]=trancs(n,m,iat,jat,aat);
    %
    %    compare the arrays (checking correctness of the transposition)
    %
    diff = norm(ic(1:n+1)-ia(1:n+1))+ norm(jc(1:ind-1)-ja(1:ind-1))+ norm(ac(1:ind-1)-aa(1:ind-1));
    fprintf('%s %1.8g \n',' difference between a and (a^T)^T: ', diff)
  %
  % 6/  matrix-matrix multiplication: standard
  %     ia,ja,aa by rows
  %     iat,jat,aat the same matrix by columns
  %  
    [id,jd,ad]=matmat0(m,n,ia,ja,aa,n,m,iat,jat,aat);
  %
  %     test correctness 
  %
    a=csr2a(m,ia,ja,aa); d=csr2a(m,id,jd,ad);
    diff=norm(full(d-a*a)); % checked in the matlab native format
    fprintf('%s %1.8g \n',' correctness test of standard CSR-CSC matmat: ', diff)
   
  %
  % 7/ fast matrix-matrix multiplication: matrices by rows
  %
    b=a; [ib,jb,ab]=a2csr(b);
    [id,jd,ad]=matmat_rr(m,n,ia,ja,aa,m,n,ib,jb,ab);
  %
  %     test correctness
  %
    a=csr2a(m,ia,ja,aa);    b=csr2a(m,ib,jb,ab);    d=csr2a(m,id,jd,ad);
    diff=norm(full(d-a*b));
    fprintf('%s %1.8g \n',' correctness test of CSR-CSR matmat: ', diff)
   
  %
  % 8/ fast matrix-matrix multiplication: first by columns, second by rows
  %
    b=a; [ib,jb,ab]=a2csr(b);
    [id,jd,ad]=matmat_cr(n,m,iat,jat,aat,m,n,ib,jb,ab);
  %                                                                                                        
  % test correctness
  %
    a=csr2a(m,ia,ja,aa); b=csr2a(m,ib,jb,ab); d=csr2a(m,id,jd,ad);
    diff=norm(full(d-a*b));
    fprintf('%s %1.8g \n',' correctness test of CSC-CSR matmat: ', diff)

    [out] =inverseAckermann(m,n)
 