    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
    n=7;
    a=sprand(n,n,0.5)+17*eye(n); % define sparse random matrix
    a=a'*a;
    a=full(a)
    asave=a;
    full(a);
    %
    %  check LU by columns using blocks
    %
    a=asave;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... column LU')
    [ll uu]=lu(a)
    ll*uu-a
    [l,u]=lu_columns(a)
    fprintf('%s %1.8g \n','     column LU: residual norm(a-l*u) = ',norm(full(asave-l*u)))
    %
    % column LDLT (just to show that the Theorem is OK, checking inside for eye, comparing the formulas)
    %
    a=asave;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... column LDLT to check Observation 3.2')
    [l,d]=ldlt_columns(a);
    fprintf('%s %1.8g \n','     column LDLT: residual norm(a-l*d*lT) = ',norm(full(asave-l*d*l')))
    %
    %  check LU by bordering
    %
    a=asave;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... bordering LU')
    [l,u]=lu_bordering(a)
    fprintf('%s %1.8g \n','     bordering LU: residual norm(a-l*u) = ',norm(full(asave-l*u)))

    rng('default')
    matrix_type=0;
    matrix_type=-2;
    matrix_type=-1;
    matrix_type=1;
    matrix_type=-1;
  %
  % 1/ read a matrix into the internal matlab format
  %
    if (matrix_type == -1) % define a small matrix directly
    %
    % define dimensions
    % 
      m=9; % number of rows
      n=m; % number of columns
      a=sprand(m,n,0.3)+eye(n); % define sparse random matrix
    elseif (matrix_type == 0) % read .mtx file
      a = mmread('./bcsstk03.mtx');
%      a = mmread('./nos3.mtx');
%      a=a+a';
%      a=a(1:7,1:7); 
%      a=a/100000;
%      a = simple2(a);
      [m n]=size(a);
    elseif (matrix_type == 1) % 
      a = poisson(2); 
      [m n]=size(a);
    elseif (matrix_type == -2) % 
      aa = rand(2,2);
      bb = zeros(2,2);
      a = [aa bb; bb aa];
      p=[1 3 2 4];
      a=a(p,p)
    end
 %
 % 2/ try different forms of LU 
 %    we assume strong regularity (no permutations needed)
 %
    [m n]=size(a);
    nsteps=m;
    %
    % we keep both M and N as parameters, although we do not use both of them here 
    %
    if( m ~= n) disp 'error'; end
 % 
 %  2a) KIJ
 %
    [l,u]=lu_kij2(m,n,nsteps,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of kij-LU: ', diff)
 % 
 %  2b) KJI
 %
    [l,u]=lu_kji(m,n,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of kji-LU: ', diff)
 % 
 %  2c) IJK
 %
    [l,u]=lu_ijk(m,n,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of ijk-LU: ', diff)
 % 
 %  2d) IKJ
 %
    [l,u]=lu_ikj(m,n,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of ikj-LU: ', diff)
 % 
 %  2e) JKI
 %
    [l,u]=lu_jki(m,n,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of jki-LU: ', diff)
 % 
 %  2f) JIK
 %
    [l,u]=lu_jik(m,n,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of jik-LU: ', diff)
 % 
 %  2g) IJK-JIK
 %
    [l,u]=lu_ijkjik(m,n,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of ijkjik-LU: ', diff)
 % 
 %  2h) JKI-IKJ
 %
    [l,u]=lu_jkiikj(m,n,a);
  %
  %     test correctness
  %
    diff=norm(full(a-l*u));
    fprintf('%s %1.8g \n','correctness test of jkiikj-LU: ', diff)



