    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
    %
    %  find the strong components
    %
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... find strong components')
    n=8;
    a = sprand(n,n,0.4); % generate a general matrix 
    a=a'; 
    full(a)
    [ia,ja]=a2cs(a);	
    if(figures) 
      figure(1);clf;
      G = digraph(a,'omitselfloops')
      plot(G,'Layout','layered')
    end
    [bins,binsizes] = conncomp(G)
    [ncomp,ic,jc] = scomp(n,ia,ja);
    [ncomp,ic,jc,preord] = scomp_recursive(n,ia,ja)
    preord;
    [ncomp,ic,jc]=scomp_recursive2(n,ia,ja)  % another - without globals, done earlier
    [bins,binsizes]=scomp2bins(n,ncomp,ic,jc) % strong components in bins
    %
    %  generate a block upper triangular matrix
    %
    nbl=5;
    density=0.2;
    maxblsize=3;
    [a,ptrb] = gen_block_upper_triangular(nbl,density,maxblsize);
    [n n]=size(a);
    a=a+20*eye(n); % regularize, for any case
    x=ones(n,1); b=a*x;
    [lub]=factorize_block_upper_triangular(nbl,ptrb,a);
    [x] = solve_block_upper_triangular(nbl,ptrb,lub,b)
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... solved a block upper triangular system')
 
    %
    % generate matrix with some blocks
    %

    nbl=4; 
    density=0.3;
    maxblsize=3;
    [a,ptrb] = gen_block_matrix(nbl,density,maxblsize);
    [n n]=size(a);
    a(3,5)=1; a(5,3)=1;
    full(a);
    p=randperm(n);
    b=a(p,p);
    full(b)
    [ib,jb,ab]=a2csr(b);
    %
    % various supervariable codes (motivated by Duff, Reid et al.)
    % (matrix must be symmetric) and permute the matrix
    %
    [s]=mc60_supervariable(n,ib,jb);
    s;
    %
    % another code to find supervariables
    %
    [s]=supervariable(n,ib,jb);
    s;
    [p]=find_supervariable_permutation(s)
    b(p,p)
    [s]=mc60_supervariable(n,ib,jb);
    fprintf('%s %1.8g \n', 'Supervariable map of a block matrix: ')
    s
    %
    % Ashcraft strategy
    %
    [nc,adjmap] = findbl(n,ib,jb)
    [perm]=adjmap2p(n,adjmap);
    full(b)
    full(b(perm,perm))
    fprintf('%s %1.8g \n', 'Ashcraft blocks: matrix dimension: ', n, 'number of blocks: ', nc)
    %
    %  blocks according to Ashcraft again (my tests)
    %
    a=[1 0 1 0 1 0; 
       0 1 0 0 1 1; 
       1 0 1 0 1 0; 
       0 0 0 1 0 0; 
       1 1 1 0 1 1; 
       0 1 0 0 1 1];  
    [n n]=size(a);
    p=randperm(n)
    b=a(p,p);
    [m n]=size(b);
    [~,invp]=sort(p);
    [ib,jb,ab]=a2csr(b)
    [nc,adjmap_1] = findbl_ash(n,ib,jb)
    b(invp,invp) % original matrix
    [perm]=adjmap2p(n,adjmap_1)
    b=b(perm,perm) % permuted matrix
    %
    [ib,jb,ab]=a2csr(b);
    [ibt,jbt,abt]=a2csr(b');
    [nc,adjmap_3] = findbl_cos(n,ib,jb,ibt,jbt,1);
    sort(adjmap_1) 
    sort(adjmap_3)
stop
    [perm]=adjmap2p(n,adjmap_3);
    full(b)
    full(b(perm,perm))
    fprintf('%s %1.8g \n', 'Saad blocks: matrix dimension: ', n, 'number of blocks: ', nc)
    a=[1 0 1 0 1 0 0;  0 1 0 0 0 1 1;  1 0 1 0 1 0 0;  0 0 0 1 0 0 0; 1 0 1 0 1 1 0;  0 1 0 0 1 1 0;  0 1 0 0 0 0 1]
    [n n]=size(a);
    [ia,ja,aa]=a2csr(a);
    [iat,jat,aat]=a2csr(a');
    tau=0.5;
    [nc,adjmap] = findbl_cos(n,ia,ja,iat,jat,tau)
    [perm]=adjmap2p(n,adjmap);
    a(perm,perm)

