    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
    %
    % construct elimination tree of a symmetric matrix
    %
    n=8;
    a=sprand(n,n,0.3);
    a=sprand(n,n,0.2)+10*eye(n); % define sparse random matrix
    a=a+a';
    [ia,ja,aa]=a2csr(a);
    fprintf('\n')
    fprintf('%s \n',' ... Sparse SPD matrix defined')
    full(a)
    chol(a);
    ans'
    %
    % standard computation of the elimination tree
    %
    [parent]=eltree(n,ia,ja,aa);
parent'
    %
    % computation of the elimination tree with path compression
    %
    [parent3]=eltree3(n,ia,ja,aa)
    fprintf('\n')
    fprintf('%s %1.8g \n', 'difference in elimination trees: ', norm(parent-parent3))
    % 
    % compute row counts (using parent)
    %
    [rowcnt]=rowl_counts(n,parent,ia,ja,aa);
rowcnt'

    sumr=0;
    for i=1:n
      sumr=sumr+rowcnt(i);
    end 
    sumr=sum(rowcnt); % matlab sum :-)
    % sumc
    % sumr
    %
    % compare the computed count with the count from matlab (native sparse format)
    %
    l=chol(a+n*eye(n)); f=l+l';
    full(f);
    rowlcnt=rowl_counts_a(l');  % Cholesky factor is in Matlab an upper triangular matrix
    fprintf('\n')
    fprintf('%s %1.8g \n', 'difference in row counts: ', norm(sumr-sum(rowlcnt)))
    %
    % compute structures of rows of the Cholesky (lower triangular) factor
    %
    [il,jl]=rowl_struct_a(l');
    [il,jl]=srtcs(n,il,jl);
    [il2,jl2]=rowl_struct(n,parent,ia,ja);
    [il2,jl2]=srtcs(n,il2,jl2);
    ll=cs2a(n,il,jl);f=ll+ll';
    full(f);
    ll2=cs2a(n,il2,jl2);f2=ll2+ll2';
    full(f2);
    full(a);
    fprintf('\n')
    fprintf('%s %1.8g \n', 'difference in row structures: ', norm(il-il2)+norm(jl-jl2))
    % 
    % compute column counts (using parent)
    %
    [colcnt]=coll_counts(n,parent,ia,ja,aa);
    sumc=0;
    for i=1:n
      sumc=sumc+colcnt(i);
    end 
    fprintf('\n')
    fprintf('%s %1.8g \n', 'difference between row and column counts: ', norm(sumr-sumc))
    % 
    % symbolic factorization (column structures, only off-diagonals)
    %
    [il,jl]=sfact(n,ia,ja,sumr)
    %
    % another symbolic factorization
    %
    [colcnt]=coll_counts(n,parent,ia,ja,aa)';
    [il3]=setup_cs_by_cols(n,colcnt);
    [ichild]=setup_children_pointers(n,parent);
    [il3,jl3]=sfact2(n,ia,ja,il,ichild);
    [il3,jl3]=srtcs(n,il3,jl3); % sort
    fprintf('\n')
    fprintf('%s %1.8g \n', 'difference in column structures: ', norm(il-il3)+norm(jl-jl3))
    %
    % add the diagonal (not in the column structures yet)
    %
    [il,jl]=insert_diagonal_cs(n,il,jl);
    [il,jl]=srtcs(n,il,jl);
    ila=cs2a(n,il,jl);fila=ila+ila';
    full(f2);
    full(fila);
    %
    % get "dual" (transpose) to parent (another and more standard description is via brothers)
    %
    [son,ison]=sonstr(n,parent)
    fprintf('\n')
    fprintf('%s \n',' ... Transpose to parent computed')
    %
    [ipt,jpt,apt]=parent2csr(n,parent)
    par=csr2a(n,ipt,jpt,apt);
    %
    figure(1);clf;
    G = digraph(par,'omitselfloops');
    plot(G,'Layout','layered','Direction','up')
    fprintf('\n')
    fprintf('%s \n',' ... Elimination tree shown')
    %
    % to see parent as a symmetric matrix:
    % transpose the output "parent" matrix to put it into the lower triangular form 
    %
    [ip,jp,ap]=trancs(n,n,ipt,jpt,apt);
    part=csr2a(n,ip,jp,ap);
    c=par+part;
    figure(2);clf;
    spy(c) 
    fprintf('\n')
    fprintf('%s \n',' ... Elimination tree shown as a pattern (symmetrized)')
    %
    % postordering of the elimination tree (can be based on DFS as well)
    %
    [postord,ipsord]=get_postord(n,parent,son,ison)
    posta=a(postord,postord); full(posta)
    %
    % postordered matrix: repeating the steps for the postordered matrix
    %
    [ia ja aa]=a2csr(posta);
    [parent]=eltree(n,ia,ja,aa); 
    [colcnt]=coll_counts(n,parent,ia,ja,aa)'; % again column counts

    [ipt,jpt,apt]=parent2csr(n,parent);
    par=csr2a(n,ipt,jpt,apt);
    %
    figure(3);clf;
    G = digraph(par,'omitselfloops');
    plot(G,'Layout','layered','Direction','up')
    fprintf('\n')
    fprintf('%s \n',' ... Elimination tree of the postordered matrix shown')
    %
    %  subtree counts
    %
    [scounts]=subtree_counts(n,parent); % parent corresponds to the postordered matrix
    fprintf('\n')
    fprintf('%s \n',' Subtree counts computed')
    %
    %  compute leaves of row subtrees
    %
    [leaves]=row_subtree_get_leaves(n,ia,ja,scounts)
    fprintf('\n')
    fprintf('%s \n',' Leaves of row subtrees computed')

    k=4;
    [parentk]=row_subtree_get(n,ia,ja,parent,k);
    [iptk,jptk,aptk]=parent2csr(k,parentk);
    park=csr2a(k,iptk,jptk,aptk);
    %
    if(never) % depicting k-th row subtree + some other vertices that do not belong to it :-)
      figure(4);clf;
      names=[];
      for i=1:k
        num(i)=i;
        names(i)=num2str(i);
      end
      str=string(num)
      names={'a1','b2','c3','d4','e5','f6','g7','h8'};
      names=str;
      G = digraph(park,names,'omitselfloops');
      GG=G;
      mark=zeros(1,k);
      for i=1:k
        if(indegree(GG,i) == 0 && outdegree(GG,i) == 0)
%        G=rmnode(G,i);
          mark(i)=1;
        end
      end 
      mark
      plot(G,'Layout','layered','Direction','up')
    end
