    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
    n=8;
    a=sprand(n,n,0.4)+10*eye(n); % define sparse random matrix
    a=a'*a;
    asave=a;
    fprintf('\n')
    fprintf('%s \n',' ... Sparse SPD matrix defined')
    %
    % in-place dense left-looking Cholesky
    %
    fprintf('\n')
    fprintf('%s \n',' ... In-place dense left-looking Cholesky')
    l=in_place_dense_left_looking_chol(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
    %
    blsize=2;
    remainder=mod(n,blsize);
    a=a(1:n-remainder,1:n-remainder); % cut the rest
    asave=a;
    [n n]=size(a);
    fprintf('%s %d %d \n',' ... Dimension, blocksize',n, blsize)
%
    fprintf('\n')
    fprintf('%s \n',' ... In-place dense left-looking panel')
    a=asave;
    l=dense_left_looking_panel(blsize,a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
%
    fprintf('\n')
    fprintf('%s \n',' ... In-place dense right-looking panel')
    a=asave;
    full(a);
    l=dense_right_looking_panel(blsize,a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
%
    fprintf('\n')
    fprintf('%s \n',' ... In-place dense right-looking block')
    l=dense_right_looking_block(blsize,a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
%
    fprintf('\n')
    fprintf('%s \n',' ... In-place dense right-looking block with tasks')
    l=dense_right_looking_block_tasks(blsize,a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
%
    fprintf('\n')
    fprintf('%s \n',' ... Simplified sparse left looking')
    l=simplified_sparse_left_looking(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
%
    fprintf('\n')
    fprintf('%s \n',' ... Simplifies sparse right looking')
    l=simplified_sparse_right_looking(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
%
    fprintf('\n')
    fprintf('%s \n',' ... Sparse up looking')
    l=sparse_up_looking(a);
    fprintf('%s %1.8g \n','     residual norm(a-l*l'') = ',norm(full(a-l*l')))
%   
    %
    % getting a postordered matrix (in-place)
    %
    n=8;
    a=sprand(n,n,0.3);
    a=sprand(n,n,0.2)+10*eye(n); % define sparse random matrix
    a=a+a';
    [ia,ja,aa]=a2csr(a);
    fprintf('\n')
    fprintf('%s \n',' ... Sparse SPD matrix defined')
    full(a)
    full(a);
    [n n]=size(a);
    posta=postorder_matrix(a);
    full(posta);
    [ia ja aa]=a2csr(posta);
    [parent]=eltree(n,ia,ja,aa); 
    fprintf('\n')
    fprintf('%s \n',' ... In-place postordering (again)')
    %
    fprintf('\n')
    fprintf('%s \n',' ... The multifrontal Cholesky')
    postal=triu(posta); % triangular part used on input
    [ia ja aa]=a2csr(postal) % conversion to CSR
    [il,jl,al]=multifrontal_cholesky(n,parent,ia,ja,aa)
    lfactor=csr2a(n,il,jl,al); 
    full(lfactor')
    full([chol(posta)]') % for checking purposes
    fprintf('\n')
    fprintf('%s %1.8g \n', 'difference in factorization (matlab versus multifrontal): ', norm(lfactor' - [chol(posta)]'))
    %


    %
    % nodal sparse Cholesky
    %
    [ia ja aa]=a2csr(postal) % converting to CSR
    l=chol(posta); 
    nrm1=norm(posta-l'*l)
    d=diag(diag(l));  l = [inv(d)*l]'; dd=d*d;
    l=l-diag(diag(l))+eye(n);
    nrm2=norm(posta-l*dd*l')
    full(posta);
    [il,jl,al] = ldlt_ll_sparse(n,ia,ja,aa);
    aldlt=[full(csr2a(n,il,jl,al))]';
    d=diag(diag(aldlt));
    ll=[aldlt*d-diag(diag(aldlt*d))+eye(n)'];
    dd=inv(d);
    nrm3=norm(posta-ll*dd*ll')
    [il,jl,al] = cholesky_ll_sparse_tasks(n,ia,ja,aa);
    achol=[full(csr2a(n,il,jl,al))]';
    nrm4=norm(posta-achol*achol')
    disp 'not finished'
    %
    % supernodal sparse Cholesky
    %
    [nsuper,snode,ssiz]=find_maximal_supernodes(n,parent,colcnt)
    [nsons]=count_sons(n,parent);
    [nsuper,snode,ssiz]=find_fundamental_supernodes(n,parent,leaves,nsons)
    full(posta);
    [colcnt]=coll_counts(n,parent,ia,ja,aa)';
    [il3]=setup_cs_by_cols(n,colcnt);
    [ichild]=setup_children_pointers(n,parent);
    [il3,jl3]=sfact2(n,ia,ja,il,ichild);
    [il3,jl3]=insert_diagonal_cs(n,il3,jl3);
    [il3,jl3]=srtcs(n,il3,jl3);
    full(posta)
    full([chol(posta)]')
    [ibl,jbl,sbl,al] = super_setup_triangular(n,nsuper,il,jl,ia,ja,aa,snode,ssiz)
snode
ssiz
    full(posta)
    full([chol(posta)]')
    [al]=extend_diagonal_blocks(n,nsuper,ssiz,sbl,il,al); 
    [ip,jp,ap] = cholesky_sll_sparse_tasks(n,nsuper,ssiz,snode,ibl,jbl,sbl,al)
%
% test by solving a system
%
    x=ones(n,1); b=posta*x;
    ssiz
    x=cholesky_super_solve(n,nsuper,ip,jp,ap,ssiz,snode,b)
huhu
    [lchol]=cssup2a(n,nsuper,ssiz,sbl,il,jl,al);
    norm(lchol-tril(posta))
    full([chol(posta)]')
    tril(posta)
    full(lchol)
    al;
    ssiz
    [al]=extend_diagonal_blocks(n,nsuper,ssiz,sbl,il,al) 
huhu
    [il,jl,al] = cholesky_ll_sparse_tasks(n,ia,ja,aa);
    %
    [is,js,as]=supernodal_cholesky(n,nsuper,ssiz,posta)
huhu
    %
    % left-looking supernodal factorization
    %  

    huhu


