    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
    n=5;
    a=sprand(n,n,0.8)+10*eye(n); % define sparse random matrix
    a=a+a';
    asave=a;
    %
    % Diagonal pivoting method with the Bunch-Parlett pivoting
    %
    a=asave;
full(a)
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... Indefinite LDLT factorization with complete pivoting, alpha =',(1+ sqrt(17))/8)                                            
    [l,d,P]=bp(a);
    ptap=P'*asave*P;
    fprintf('%s %1.8g \n','     residual norm using the complete Bunch-Parlett pivoting = ',norm(full(ptap-l*d*l')))
    %
    % Diagonal pivoting method with the Bunch-Kaufmann pivoting
    %
    a=asave;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... Indefinite LDLT factorization with Bunch-Kaufmann pivoting, alpha =',(1+ sqrt(17))/8)
    [l,d,P]=bk(a);
    ptap=P'*asave*P;
    fprintf('%s %1.8g \n','     residual norm using the Bunch-Kaufmann pivoting = ',norm(full(ptap-l*d*l')))
    %
    % Diagonal pivoting method with the rook pivoting
    %
    a=asave;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... Indefinite LDLT factorization with the rook pivoting, alpha =',(1+ sqrt(17))/8)
    [l,d,P]=brook(a);
    ptap=P'*asave*P;
    fprintf('%s %1.8g \n','     residual norm using the rook pivoting = ',norm(full(ptap-l*d*l')))
    %
    %  Simple sparse indefinite factorization
    %
    a=asave;
    tau=0.1;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... Simple sparse indefinite LDLT factorization, tau =',tau)
    [l d P]=sparse_indefinite(a,tau)
    ptap=P'*asave*P;
    fprintf('%s %1.8g \n','     residual norm using simple sparse indefinite factorization = ',norm(full(ptap-l*d*l')))
    
    %
    %  ILU(tau) Crout algorithm + iterative refinement
    %
    n=15;
    a=sprand(n,n,0.8)+10*eye(n); % define sparse random matrix
    asave=a;
    a=asave;
    tau=0.1;
    fprintf('\n')
    fprintf('%s  %1.8g \n',' ... Crout ILU(tau), tau =',tau)
    [l,u]=ilu_crout(a,tau);
    fprintf('%s %1.8g \n','     Crout ILU: residual norm(a-l*u) = ',norm(full(asave-l*u)))
    x0=ones(n,1);b=asave*x0;nsteps=3;
    [x]=iterative_refinement(a,l,u,b,nsteps);
    fprintf('%s %d %1.8g \n','     Iterative refinement: , nsteps = ', nsteps, norm(x0-x))
    %
    %  Special indefinite matrices that avoid pivoting
    %
    n1=5;n2=3;g=sprand(n1,n1,0.4)+n1*eye(n1); % define special sparse random matrix
    g=g*g' 
    r=sprand(n2,n1,0.3);
    b=sprand(n2,n2,0.2);
    a=full([g r'; r -b])
    P=eye(n1+n2);
    ind1=1;ind2=n1+1;
    ind=1
    for i=1:n2
      p(ind)=ind1;
      p(ind+1)=ind2;
      ind1=ind1+1;ind2=ind2+1;ind=ind+2;
    end
    for i=n2+1:n1
      p(ind)=ind1;
      ind1=ind1+1;ind=ind+1;
    end
    full(a(p,p)) % permuted saddle-point matrix 
    fprintf('%s %d %1.8g \n','     This is the permuted matrix that can be factorized by 2x2 and 1x1 pivots (we do not do it here)')
    %
    %  Find the equilibration scaling in the infinity norm
    %
    a=asave;
    tol=0.00001;
    nsteps=10;
    fprintf('\n')
    fprintf('%s  %1.8g %d \n',' ... Equilibration scaling, tolerance, nsteps =',tol,nsteps)
    [sr,sc] = equilibration_scaling(a,nsteps,tol);
    % sr*a*sc
    fprintf('%s %1.8g \n','     departure from unit matrix = ',norm(full(eye(n)-sr*a*sc),inf))

