    % Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
    n=16;
    a=sprand(n,n,0.8)+10*eye(n); % define sparse random matrix
    a=a+a';
    asave=a;
    
    full(a);
    %
    %  Gauss-Seidel iterative method (preconditioning)
    %
    a=asave;
    rtol=1.0d-8;
    fprintf('\n')
    fprintf('%s %1.8g \n',' ... Jacobi iterative method, rtol = ',rtol)
    b=ones(n,1); maxit=1000;
    [x,relres,iter,info] = jacobi(a,b,rtol,maxit);
    fprintf('%s %d %1.8g %d \n','     Jacobi: info, relative residual norm, its = ', info,relres,iter)
    %
    %  Gauss-Seidel iterative method (preconditioning)
    %
    a=asave;
    rtol=1.0d-8;
    fprintf('\n')
    fprintf('%s %1.8g \n',' ... Gauss-Seidel iterative method, rtol = ',rtol)
    b=ones(n,1); maxit=1000;
    [x,relres,iter,info] = gseidel(a,b,rtol,maxit);
    fprintf('%s %d %1.8g %d \n','     Gauss-Seidel: info, relative residual norm, its = ', info,relres,iter)
    %
    %  SOR iterative method (preconditioning)
    %
    a=asave;
    rtol=1.0d-8;
    fprintf('\n')
    fprintf('%s %1.8g \n',' ... SOR iterative method, rtol = ',rtol)
    b=ones(n,1); maxit=1000;
    omega=1.0d0;
    [x,relres,iter,info] = sor(a,b,omega,rtol,maxit);
    fprintf('%s %d %1.8g %1.8g %d \n','     SOR: info, relative residual norm, its = ', info,omega,relres,iter)
    %
    %  D-ILU preconditioning and Eisenstat trick
    %
    a=asave;
    rtol=1.0d-8;
    fprintf('\n')
    fprintf('%s \n',' ... D-ILU preconditioniong (updates only diagonal entries)')
    b=ones(n,1); maxit=1000;
    omega=1.0d0;
    [l,d,u]=d_ilu(a);
    da=diag(diag(a));la=-tril(a)+da; ua=-triu(a)+da;  da-la-ua-a;
    % m and mm should be the same :-)
    m=(l)*inv(d)*(u); mm=(d-la)*inv(d)*(d-ua);
    lbar=inv(d)*la; ubar=inv(d)*ua; abar=inv(d)*a;bbar=inv(eye(n)-lbar)*inv(d)*b;
    z=(1:n)';
    % the following two vectors should be the same (with and without Eisenstat trick)
    w1=inv(eye(n)-ubar)*z; w2=inv(eye(n)-lbar)*((inv(d)*da-2*eye(n))*w1 +z); w=w1+w2;
    ww=inv(eye(n)-lbar)*abar*inv(eye(n)-ubar)*z;
    fprintf('%s %1.8g \n',' ... checking Eisenstat trick (with and without - outside the iterative method), comp_error = ',norm(w-ww))
    x=ones(n,1);b=asave*x;m1=l*inv(d);m2=u;rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d\n','     GMRES/D-ILU: flag, rtol, its = ',flag,rtol,iter)
    %
    % truncated Neumann series preconditioner
    %
    a=asave;
    rtol=1.0d-8;
    fprintf('\n')
    fprintf('%s \n',' ... D-ILU preconditioniong (updates only diagonal entries)')
    b=ones(n,1); maxit=1000;
    omega=1.0d0;
    [l,d,u]=d_ilu(a);
    mbar=(l)*inv(d)*(u);
    omega=10;
    k=2;
    [minv]=truncated_neumann(a,omega,mbar,k);
    x=ones(n,1);b=asave*x;m1=inv(minv);m2=eye(n);rtol=1.0d-8;maxit=1000;x0=zeros(n,1);restrt=10;
    [x,flag,relres,iter] = gmr(asave,b,rtol,maxit,restrt,m1,m2,x0);
    fprintf('%s %d %1.8g %d %d \n','     GMRES/truncated Neumann: flag, rtol, no. of terms, its = ',flag,rtol,k,iter)

