%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% inverse by bordering
%
% function [Z,D,W] = aibn(A,tol);
%
% purpose:
%   Computes the factorization 
%          ZD^{-1}W^T  \approx inv(A) 
%   very simple dense code.
%
% input:
%   A:    matrix to be factored;
%   tol:  drop tolerance for elements in U and V factors;
%
% output:
%   Z,D,W:  AIB factors.
%
% history:
%   the code by Mirek Tuma, 2021.
%
  function [Z,D,W] = aibn(A,tol);
  %
  % check and complete parameters
  %
  if (nargin <= 1) tol = 0; end
  %
  % basic initializations
  %
  [n,n] = size(A);
  I=eye(n);
  Z=zeros(n,n);W=Z;D=Z;
  Z(1,1)=1;W(1,1)=1;D(1,1)=A(1,1);
  %
  % the loop
  %
  for j=2:n
    %
    zj=-Z(1:j-1,1:j-1)*inv(D(1:j-1,1:j-1))*W(1:j-1,1:j-1)'*A(j,1:j-1)';
    wj=-W(1:j-1,1:j-1)*inv(D(1:j-1,1:j-1))*Z(1:j-1,1:j-1)'*A(1:j-1,j);
    dj=A(j,j) + A(j,1:j-1)*wj + zj'*A(1:j-1,j) + zj'*A(1:j-1,1:j-1)*wj;
    djhat=A(j,j)+A(j,1:j-1)*zj;
    for i=1:j-1
      if(abs(zj(i)) < tol)
        zj(i)=0.0;
      end
    end
    Z(1:j-1,j)=zj;Z(j,j)=1;
    for i=1:j-1
      if(abs(wj(i)) < tol)
        wj(i)=0.0;
      end
    end
    W(1:j-1,j)=wj;W(j,j)=1;
    D(j,j)=dj;
    %
    %
  end
   
  
  %
  % test the result
  %
  %  norm(A-inv(Z*inv(D)*W'))


