%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% function [Z,D] = ainvl(A,tol);
%
% left-looking ainv decomposition
%
%
% purpose:
%   Computes the factorization 
%         ZD^{-1}Z^T  \approx inv(A) 
%   of the symmetric and positive definite matrix A.
%   simple dense code.
%
% input:
%   A:    matrix to be factored;
%   tol:  drop tolerance for elements in U and V factors;
%
% output:
%   Z,D:  AINV factors.
%
% history:
%   the code by Mirek Tuma, 2003.
%
  function [Z,D] = ainvl(A,tol);
%
% check and complete parameters
%
  if (nargin <= 1) tol = 0; end
%
% basic initializations
%
  [n,n] = size(A);
  Z = eye(n);
  D = eye(n);
  I=eye(n);
  %
  % the loop
  %
  for j = 1:n 
    for k=1:j-1
      %
      % find the multipliers
      %
      alpha = inv(D(k,k))*A(k,:)*Z(:,j);
      %
      % update the remaining columns
      %
      Z(:,j) = Z(:,j) - alpha*Z(:,k); % this is the left-looking AINV
      %
      % dropping
      %
      for l=1:j-1 % j-th diagonal entry should be kept
        if(abs(Z(l,j)) < tol)
          Z(l,j)=0.0;
        end
      end
    end
    dj = A(j,1:n)*Z(:,j);
    D(j,j) = dj;
    % 
  end
%

