%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% function [Z] = ainvl2(A,tol);
%
% square-root left-looking ainv decomposition
%
%
%
% purpose:
%   Computes the factorization 
%         ZD^{-1}Z^T  \approx inv(A) 
%   of the symmetric and positive definite matrix A.
%   simple dense code.
%
% input:
%   A:    matrix to be factored;
%   tol:  drop tolerance for elements in U and V factors;
%
% output:
%   Z,D:  AINV factors.
%
% history:
%   the code by Mirek Tuma, 2003.
%
  function [Z,U] = ainvl2(A,tol);
%
% check and complete parameters
%
  if (nargin <= 1) tol = 0; end
%
% basic initializations
%
  [n,n] = size(A);
  Z = eye(n);
  U=zeros(n,n);
  pivot=diag(A);
  D = eye(n);
  I=eye(n);
  %
  % the loop
  %
  for i = 1:n 
    %
    % diagonal pivoting (unfinished)
    %
    if (i > 1) 
      for j=i:n
        update=(A(j,:)*Z(:,i-1));
        pivot(j)=pivot(j)-update*update;
      end
    end
    pivot;
    %
    for j=1:i-1
      %
      % find the multipliers
      %
      U(j,i) = I(:,i)'*A*Z(:,j);
      %
      % update the remaining columns
      %
      Z(:,i) = Z(:,i) - Z(:,j)*U(j,i); % not checked whether correct AINV
      %
      % dropping
      %
      for k=1:n
        if(abs(Z(k,i)) < tol)
          Z(k,i)=0.0;
        end
      end
    end
    U(i,i) = A(i,:)*Z(:,i);
    U(i,i) = sqrt(U(i,i));
    Z(1:i,i)=Z(1:i,i)/U(i,i);
    % 
  end
%
% test the result
%
%  norm(A-inv(Z*Z'))  


