%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% right-looking nonzymmetric ainv preconditioner/factorization
%
% function [Z,D,W] = ainvr(A,tol,V);
%
% purpose:
%   Computes the factorization 
%          ZD^{-1}W^T  \approx inv(A) 
%   very simple dense code.
%
% input:
%   A:    matrix to be factored;
%   tol:  drop tolerance for elements in U and V factors;
%
% output:
%   Z,D,W:  AINV factors.
%
% history:
%   the code by Mirek Tuma, 2003.
%
  function [Z,D,W] = ainvnr(A,tol);
  %
  % check and complete parameters
  %
  if (nargin <= 1) tol = 0; end
  %
  % basic initializations
  %
  [n,n] = size(A);
  D = sparse(n,n);
  D = eye(n);
  I=eye(n);
  Z=eye(n);W=eye(n);
  %
  % the loop
  %
  for j=1:n
    %
    dj=A(1:n,j)'*Z(:,j);
    %
    % update the remaining columns
    %
    for k = j+1:n
      alpha=A(1:n,j)'*Z(:,k)*inv(dj);
      Z(:,k) = Z(:,k) - alpha*Z(:,j);
      for i=1:j-1
        if(abs(Z(i,k)) < tol)
          Z(i,k)=0.0;
        end
      end
      beta=A(j,1:n)*W(:,k)*inv(dj);
      W(:,k) = W(:,k) - beta*W(:,j);
      for i=1:j-1
        if(abs(W(i,k)) < tol)
          W(i,k)=0.0;
        end
      end
    end
    D(j,j) = dj;
    %
    %
  end
  %
  % test the result
  %
%  norm(A-inv(Z*inv(D)*W'))


