%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% right-looking SPD ainv preconditioner/factorization
%
% function [Z,D] = ainvr(A,tol,V);
%
% purpose:
%   Computes the factorization 
%          ZD^{-1}Z^T  \approx inv(A) 
%   of the symmetric and positive definite matrix A.
%   very simple dense code.
%
% input:
%   A:    matrix to be factored;
%   tol:  drop tolerance for elements in U and V factors;
%   V:    initial values for Z.
%
% output:
%   Z,D:  AINV factors.
%
% history:
%   the code by Mirek Tuma, 2003.
%
  function [Z,D] = ainvr(A,tol);
  %
  % check and complete parameters
  %
  if (nargin <= 1) tol = 0; end
  %
  % basic initializations
  %
  [n,n] = size(A);
  Z = eye(n);
  D = eye(n);
  %
  % the loop
  %
  for j=1:n
    %
    dj = A(j,1:n)*Z(:,j);
    %
    % update the remaining columns
    %
    for k = j+1:n
      alpha=inv(dj)*A(j,1:n)*Z(:,k);
      Z(:,k) = Z(:,k) - alpha*Z(:,j);
    end
    D(j,j) = dj;
    %
    % dropping
    %
    pattern=( abs(Z(:,j)) >= tol );
    pattern(j)=1;  % Keep the diagonal entries
    Z(:,j)=Z(:,j) .* pattern;
    %
    % increase loop index
    %
  end
  %

