   
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
%
   function [rperm cperm u v srank]=bpmatch_a_la_mc64(m,n,ia,ja,aa)
   %
   % maximum weight matching for bipartite graphs (between rows (A) and columns (B)
   % primal-dual algorithm
   % motivated by: algorithms by M.L.Salavatipour, code by David Gleich, Schrijver: Combinatorial optimization
   %
   tolerance=1.0d-10;
   for i=1:ia(m+1)-1
     temp=aa(i);
     if(abs(temp) < 1.0d-15) aa(i)=inf; end
   end
   a=csr2a(n,ia,ja,aa);
   full(a)
   b=[a eye(m)];
   full(b)
   [ia,ja,aa]=a2csr(b);
   for i=1:ia(m+1)-1
     k=ja(i);
     if(k > n) aa(i)=-inf; end
     if(aa(i) == inf) aa(i) =0; end
   end
   mate_column = zeros(1,m);   % column to which are matched rows
   mate_row = zeros(1,m+n); % includes slack (dummy) column variables
   %
   % set up trivial feasible solution for the weighted vertex cover
   %
   u=zeros(1,m); 
   for i=1:m
     jstrt=ia(i);
     jstop=ia(i+1)-1;
     for j=jstrt:jstop
       if (aa(j) > u(i))
         u(i)=aa(j); 
       end
     end
   end
   v=zeros(1,m+n);
   u
   v
   %
   % initialize vertices in the queue
   %
   queue = zeros(1,m);
   vertex = zeros(1,m+n);
   previous=zeros(1,m+n);
   % mate_column and mate_row are both 0, which indicates no matches
   i=1;
   %
   % for each row index i (from A) try to find a column vertex (from B)
   % such that it can be inserted into the matching
   % if there is no augmenting path starting from i and searching all paths, then
   % a dummy (slack) vertex of B is used for the matching
   %
   ind = 0;
   tail = 0;
   %
   for i=1:m % for each row vertex either new real or dummy edge is found
     %
     success=0; % not found anything yet
     while (success == 0)
      i
      % 
      % clear previous marks
      %
      for j=1:ind
        previous(vertex(j))=0; 
      end
      %
      % initialize queue by i
      %
      if (ind > 0) vertex(1:ind)=zeros(1,ind); end % not needed - for clarity in degugging
      if (tail > 0) queue(1:tail)=zeros(1,tail); end % not needed - for clarity in debugging
      ind=0;
      head=1; 
      tail=1;
      queue(head)=i 
      while (head <= tail) && (mate_column(i)==0)    % this is a row exposed node that should be matched
        k=queue(head)
        for ii=ia(k):ia(k+1)-1
          j = ja(ii)
          huhu1=u(k)+v(j)
          huhu2=aa(ii)
          if (aa(ii) + tolerance < u(k)+v(j))
            disp 'pokracuji'
            continue; % skip the edge since it does not belong to the equality graph G_y
          end 
          prevj=previous(j)
          if (previous(j)==0) % not yet in the queue (that stores the search tree)
            tail=tail+1 
            queue(tail)=mate_row(j)
            previous(j)=k
            ind=ind+1 % j added into the queue
            vertex(ind)=j
            if (mate_row(j) == 0)
               while (j ~= 0) % standard exchange between M and non M on the augmenting path
                 mate_row(j)=previous(j) % edges in D directed from coumns (B) to rows (A)
                 k=previous(j)
                 jj=mate_column(k)
                 mate_column(k)=j
                 j=jj
               end
               disp 'br'
               break; % matching is increased (mate_column(i) has been defined, jumps to the increase of head)
               % stop construct the queue
            end
          end
        end
        head=head+1
      end
      if(mate_column(i) ~= 0) 
        success=1;
      else 
        % i this case, variables are updated since the path has not been found yet
        % find minimum theta for nodes accessible from an exposed node of A (nodes in L)
        %
        theta=inf
        for j=1:head-1
          node=queue(j);
          for ii=ia(node):ia(node+1)-1  % its neighbors (in L)
            neighbor=ja(ii); % t2 belongs to B \setminus L
            if (previous(neighbor) == 0) % belongs to A  \cap L
              if(u(node) + v(neighbor) - aa(ii) < theta) % looking for minimum
                theta = u(node) + v(neighbor) - aa(ii); % redefine theta
              end
            end
          end
        end
        %
        % decrease dual variables for left (row) nodes
        %
        for j=1:head-1
          u(queue(j)) = u(queue(j)) - theta; 
        end
        u
        %
        % increase dual variables for right (column) nodes
        %
        for j=1:ind
          v(vertex(j)) = v(vertex(j)) + theta; 
        end
        v
      end
    end  % while loop 
    %
    % size of the equality graph
    %
      esize=0;
      for i=1:m
        jstrt=ia(i);
        jstop=ia(i+1)-1;
        for j=jstrt:jstop
          k=ja(j); 
          temp=aa(j)-u(i)-v(k);
          if(abs(temp) < 1.0d-10) esize=esize+1; end
        end
      end
      esize
       
  end
  %
  % output only those that are not slacks
  %
  ind=0;
  for i=1:m
    if(mate_column(i) <= n)
      ind=ind+1;
      rperm(ind)=i;
      cperm(ind)=mate_column(i);
    end
  end
  srank=ind;




