%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% find strong components of a graph (given in ia,ja)
% based on Tarjan, 1972, p.157
% non-recursive version that uses an explicit stack
% based on DFS with lowlink info
%
        function [ncomp,ic,jc]=scomp(n,ia,ja)
%
        ic(1)=1;
        iaw=zeros(n+1,1);
	ncomp=0;
	last=n; 
	sfree=1; % points to a free stack position
	for i=1:n
          lowlink(i)=n+1;
          iaw(i)=ia(i);
          reached(i)=0;
	end
	for i=1:n % for all nodes;
          if (reached(i)~=0) continue; end 
	  node=i;
	  parent(node)=-1; % root of some subgraph
	  cc=1; 
          while (cc == 1) % loop of new nodes that can be put on the stack
            cc=0;
            reached(node)=1;
            lowlink(node)=sfree;
            stack(sfree)=node;  % push
            istack(node)=sfree; % inverse stack info
            sfree=sfree+1; % stack pointer
            while (node > 0)
              jstrt=iaw(node);
              jstop=ia(node+1)-1;
              for j=jstrt:jstop
	        k=ja(j);
                if (reached(k) ~= 0) % reached node
                  if (lowlink(k)<lowlink(node)) % redefine reachability
                    lowlink(node)=lowlink(k);
                  end 
                else
                  parent(k)=node;
                  iaw(node)=j+1;
                  node=k;
                  cc=1;
                  break; % should be double break (based on this value)
                end 
              end 
	      if(cc==1) break; end
              iaw(node)=ia(node+1);
              %
	      if(lowlink(node)==istack(node)) 
                %
                % a strong component found
                % (Lemma 12, Tarjan (1972))
                %                                           
	        ncomp=ncomp+1;
                ic(ncomp)=n-last+1;
                for j=sfree-1:-1:istack(node);
                  % all nodes from stack between the 
                  % last occupied position and position of the current node 
                  stack(last)=stack(j);
                  lowlink(stack(last))=n+1;
                  last=last-1;
                end 
                sfree=istack(node); % component has been popped out of the stack
              end 
              %
              % backtrack
              %
              backtrack=node;
              node=parent(backtrack);
              if(node > 0) % if node is on the stack of the nodes   
                if(lowlink(backtrack) < lowlink(node)); % frond or cross-link found
                  lowlink(node)=lowlink(backtrack);
                end 
              end
	    end % while (go to the list of available neighbors)
	  end % while (cc==1)
	end % for i=1:n

        ic(ncomp+1)=n+1;
	for i=1:n % reversing the order (it could be stored directly at the memory expense)
          jc(i)=stack(n-i+1);
	end
