
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% ic(0) decomposition.
%
%   function [l]=ic0_chol(A)
%
% purpose:
%   Computes the ic(0) decomposition (square-root).
%   Right-looking code.
%
% input:
%   A:   matrix to be decomposed
%                                                 
% output:          
%   l,d: factors such that a \approx l*d*l'
%
% history:
%   the code by Mirek Tuma, Prague, 19/11/2005.
%
  function [l]=ic0_chol(A)
  %
  % basic initializations   
  %                   
  [n,n]=size(A);
  a=A;
  %
  % define the sparsity pattern for ic(0)
  %                   
  pattern=eye(n);
  pattern=( abs(A) > 0 );
  %
  % initialize l,d
  %
  d=sparse(eye(n));
%
% submaticova formulace (kij rozklad)
%
for k=1:n
  d(k,k)=sqrt(a(k,k));
  for i=k:n
    l(i,k)=a(i,k)/d(k,k);
  end
  for i=k+1:n
    for j=k+1:n
      if(a(i,j) ~= 0)
        a(i,j)=a(i,j)-l(i,k)*l(j,k);
      end;
    end
  end
 end
%
norm(A-l*l',inf);
%
