%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% incomplete left looking Cholesky / LDL'
% standard absolute dropping
% only maximum of lsize of largest entries 
% of each column is stored
% Tismenetsky update
% part of entries still dropped
%
  function [l,d]=icl_ad_me_ti(A,tol,lsize,rsize)
%
% check and complete parameters
%
  if (nargin <= 3) rsize = 1000; end  % no restriction default
  if (nargin <= 2) lsize = 1000; end  % no restriction default
  if (nargin <= 1) tol = 0; end

%
% basic initializations   
% 
  A=full(A);                  
  [n,n]=size(A);
  a=A;
%
  l=eye(n);
  r=zeros(n,n);
  d=eye(n);
%
% sloupcova formulace (jik rozklad)
%
  l=eye(n);
  d(1,1)=a(1,1);
 
%
  for j=1:n
%
    j;
    for k=1:j-1
      for i=j:n
        a(i,j)=a(i,j)-l(i,k)*d(k,k)*l(j,k);
      end
      for i=j:n
        a(i,j)=a(i,j)-l(i,k)*d(k,k)*r(j,k);
      end
      for i=j:n
        a(i,j)=a(i,j)-r(i,k)*d(k,k)*l(j,k);
      end
      for i=j:n
        a(i,j)=a(i,j)-r(i,k)*d(k,k)*r(j,k);
      end
    end
  %
    d(j,j)=a(j,j);
  %
  % define L
  %
    for i=j+1:n
      l(i,j)=a(i,j);
    end
  %
  % standard absolute dropping
  % next steps of Cholesky will use the sparsified columns of L
  %
    for i=j+1:n
      if(abs(l(i,j)/d(j,j)) < tol)
        l(i,j)=0.0;
      end
    end
  %
  % memory restriction: keep just at most lsize offdiagonals
  % in each column
  %
  % 1/ sort the entries by magnitudes
  %
    j;
    if (j < n) 
      v=l(j+1:n,j);
      av=abs(v);
      av;
      [y,iy]=sort(av,1,'descend');
      y;
      iy;
    end
    for i=j+1:n
      y(i-j)=v(iy(i-j));
    end
    y;
  %
  % re-initialize the j-th column of L
  %
    for i=j+1:n
      l(i,j)=0.0;
    end
  %
  % 2/ keep just at most lsize nonzeros,
  %    next (at most) rsize nonzeros are used for the Tismenetsky update
  %
    lmost=min(lsize,n-j);
    for i=1:lmost
      l(j+iy(i),j)=y(i);
    end
    rmost=min(lsize+rsize,n-j);
    if (rmost > lmost)    
      for i=lmost+1:rmost
        r(j+iy(i),j)=y(i);
      end 
    end 
  %
  % scale column j to get a column of L and R
  %
    for i=j+1:n
      l(i,j)=l(i,j)/d(j,j);
    end
    l(j,j)=1;
    for i=j+1:n
      r(i,j)=r(i,j)/d(j,j);
    end
  end
  l*d;
  r*d;
  l=l+r;
%
% get the error norm of the decomposition
%
  norm(A-l*d*l');
%
%
