%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% estimate 1-norm of the i-th row of inv(l)
%
  function [v,vnorm,vplus,vminus,vplusnorm,vminusnorm,etal]= ...
     triest(n,i,l,v,vnorm,vplus,vminus,dropvar)
  %
  % basic initialization   
  %
  half=0.5;
  if(dropvar == 1)  % algorithms 5,6; Bollhofer, LAA, 2001
%
%   decision based on the full 1-norm (adding a multiple of the 
%   column of L and then computing norm of the new vector) 
%
    i;
    if(i==1) 
      v=zeros(n,1);
      vplus=zeros(n,1);
      vminus=zeros(n,1);
    end
    muplus=1-v(i);
    muminus=-1-v(i);
    vplus(i+1:n)=v(i+1:n)+l(i+1:n,i)*muplus;
    vminus(i+1:n)=v(i+1:n)+l(i+1:n,i)*muminus;
    vplus1norm=norm(vplus(i+1:n),1);
    vminus1norm=norm(vminus(i+1:n),1);
%
    if (abs(muplus)+vplus1norm > abs(muminus)+vminus1norm) 
      xi=muplus;
      v(i+1:n)=vplus(i+1:n);
    else
      xi=muminus;
      v(i+1:n)=vminus(i+1:n);
    end
    etal=abs(xi);
    vplus;
    vminus;
    v';
    muplus;
    muminus;
    norm(vplus(i+1:n),1);
    vplus1norm;
    norm(vminus(i+1:n),1);
    vminus1norm;
    vplusnorm=vplus1norm;
    vminusnorm=vminus1norm;
  elseif (dropvar == 2) 
%
%   decision based on the contribution 1-norm
%
    if(i==1) 
      v=zeros(n,1);
      vplus=zeros(n,1);
      vminus=zeros(n,1);
    end
    muplus=1-v(i);
    muminus=-1-v(i);
%
%   add just contributions
%
    addplus(i+1:n)=l(i+1:n,i)*muplus;
    addminus(i+1:n)=l(i+1:n,i)*muminus;
    addplus1norm=norm(addplus(i+1:n),1);
    addminus1norm=norm(addminus(i+1:n),1);
%
%   
    if (addplus1norm > addminus1norm) 
      xi=muplus;
      v(i+1:n)=addplus(i+1:n)'+v(i+1:n);
    else
      xi=muminus;
      v(i+1:n)=addminus(i+1:n)'+v(i+1:n);
    end
    etal=abs(xi);
    addplus;
    addminus;
    v';
    muplus;
    muminus;
    norm(addplus(i+1:n),1);
    norm(addminus(i+1:n),1);
    vplusnorm=addplus1norm;
    vminusnorm=addminus1norm;
  elseif (dropvar == 3) 
%
%   Matthias technical report
%
%   add just contributions; for example report, algorithm 5
%   (uses variables denoted by p for the vectors)
%
%
    if(i==1) 
      v=zeros(n,1);
      vplus=zeros(n,1);
      vminus=zeros(n,1);
      vnorm=0;
      vplusnorm=0;
      vminusnorm=0;
    end
    vi=v(i);
    muplus=1-v(i)
    muminus=-1-v(i)
    step=i;
    vplusnorm=vnorm;
    vminusnorm=vnorm;
    for j=i+1:n
       if(abs(l(j,i)) > 0)
         vplus(j)=v(j)+l(j,i)*muplus;
         vminus(j)=v(j)+l(j,i)*muminus;
         vplusnorm=vplusnorm-abs(v(j))+abs(v(j)+l(j,i)*muplus);
         vminusnorm=vminusnorm-abs(v(j))+abs(v(j)+l(j,i)*muminus);
       end
    end
    vplus;
    vminus;
%   
    if (abs(muplus)+vplusnorm > abs(muminus)+vminusnorm) 
      xi=muplus;
      vnorm=vplusnorm;
      for j=i+1:n
        if(abs(l(j,i)) > 0)
          v(j)=vplus(j);
          vminus(j)=vplus(j);
         end
      end
    else
      xi=muminus;
      vnorm=vminusnorm;
      for j=i+1:n
        if(abs(l(j,i)) > 0)
          v(j)=vminus(j);
          vplus(j)=vminus(j);
        end
      end
    end
    etal=abs(xi);
    vplus;
    vminus;
    v;
  elseif (dropvar == 4) 
%
%   Algorithm 1, Bollhofer 2003
%
    if(i==1) 
      v=zeros(n,1);
      vplus=zeros(n,1);
      vminus=zeros(n,1);
      vnorm=0;
      vplusnorm=0;
      vminusnorm=0;
      for j=i+1:n 
        v(j)=v(j)+l(j,1);
      end
      etal=1;
    else
      vi=v(i);
      muplus=1-v(i);
      muminus=-1-v(i);
%
%   add just contributions
%
      step=i;
      vplusnorm=0;
      vminusnorm=0;
      for j=i+1:n
         if(abs(l(j,i)) > 0)
           vplusnorm=vplusnorm+abs(v(j)+l(j,i)*muplus);
           vminusnorm=vminusnorm+abs(v(j)+l(j,i)*muminus);
         end
      end
%   
      if (vplusnorm > vminusnorm) 
        xi=muplus;
        vnorm=vplusnorm;
        sig=1;
      else
        xi=muminus;
        vnorm=vminusnorm;
        sig=-1;
      end
%
      for j=i+1:n
        if(abs(l(j,i)) > 0)
          if (sig == 1) 
            v(j)=v(j)+l(j,i)*muplus;
          else
            v(j)=v(j)+l(j,i)*muminus;
          end
        end
      end
      v;
      etal=max(abs(muplus),abs(muminus));
    end
  elseif (dropvar == 5) 
%
%   Algorithm 2, Bollhofer 2003: counting how many components
%   increase and decrease
%
    if(i==1) 
      v=zeros(n,1);
      for j=i+1:n 
        v(j)=v(j)+l(j,1);
      end
      etal=1;
    else
      vi=v(i);
      muplus=1-v(i);
      muminus=-1-v(i);
      nplus=0;
      nminus=0;
%
%   add just contributions
%
      step=i;
      for j=i+1:n
         if(abs(l(j,i)) > 0)
           if (abs(v(j)+l(j,i)*muplus) > max(2*abs(v(j)),half)) 
             nplus=nplus+1;
           end 
           if ( max(2*(abs(v(j)+l(j,i)*muplus)),half) < abs(v(j)) ) 
             nplus=nplus-1;
           end 
           if (abs(v(j)+l(j,i)*muminus) > max(2*abs(v(j)),half)) 
             nminus=nminus+1;
           end
           if ( max(2*(abs(v(j)+l(j,i)*muminus)),half) < abs(v(j)) ) 
             nminus=nminus-1;
           end
         end
      end
      if(nplus == 0 & nminus == 0)
        vplusnorm=0;
        vminusnorm=0;
        for j=i+1:n
          if(abs(l(j,i)) > 0)
            vplusnorm=vplusnorm+abs(v(j)+l(j,i)*muplus);
            vminusnorm=vminusnorm+abs(v(j)+l(j,i)*muminus);
          end
        end
        if (vplusnorm > vminusnorm) 
          xi=muplus;
          vnorm=vplusnorm;
          sig=1;
        else
          xi=muminus;
          vnorm=vminusnorm;
          sig=-1;
        end
      elseif (nplus > nminus) 
        xi=muplus;
        sig=1;
      else
        xi=muminus;
        sig=-1;
      end
%
      for j=i+1:n
        if(abs(l(j,i)) > 0)
          if (sig == 1) 
            v(j)=v(j)+l(j,i)*muplus;
          else
            v(j)=v(j)+l(j,i)*muminus;
          end
        end
      end
      v;
      etal=max(abs(muplus),abs(muminus));
    end
    vplusnorm=etal;
    vminusnorm=etal;
    vnorm=etal;
  end

