%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% ILU: right-looking version (kij)
% directly set L and U
% Ajiz-Jennings
%
  function [l,u]=ilu_kij_aj_pattern(a,tau,omega,pattern)
%
  [m n]=size(a);
  if(m ~= n) error; end 
% 
  lu=zeros(n,n);
  for i=1:n
    for j=1:n
      if(pattern(i,j) ~= 0) lu(i,j)=a(i,j); end
    end
  end
  l=tril(lu)-diag(diag(lu))+eye(n); u=triu(lu);
  for k=1:n-1
    for i=k+1:n
       l(i,k)=l(i,k)*inv(u(k,k));
       for j=i:n
         u(i,j)=u(i,j)-l(i,k)*u(k,j);
         if ((abs(u(i,j)) < tau || pattern(i,j) == 0) && u(i,j) ~= 0 && i ~= j) 
           %
           %     Ajiz-Jennings (AJ) modification 
           %
           ro = sqrt(u(i,i)/u(j,j));
           u(i,i) = u(i,i) + omega*ro*abs(u(i,j));
           u(j,j) = u(j,j) + omega*(1/ro)*abs(u(i,j));
           u(i,j) = 0;
         end
       end % j
       for j=k+1:i-1
         l(i,j)=l(i,j)-l(i,k)*u(k,j);
         if ( (abs(l(i,j)) < tau || pattern(i,j) == 0)  && l(i,j) ~= 0) 
           %
           %     Ajiz-Jennings (AJ) modification 
           %
           ro = sqrt(u(i,i)/u(j,j));
           u(i,i) = u(i,i) + omega*ro*abs(l(i,j));
           u(j,j) = u(j,j) + omega*(1/ro)*abs(l(i,j));
           l(i,j) = 0;
        end
       end % j

     end % i
   end % k
