%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
%
% conjugate gradients
%
  function [x,relres,backerr,iter,info]=cg_CSR(csra,rhs,tol,maxit,precond,prtyp,xe,x0)
%
  if nargin == 7, n=precond.nrows; x0=zeros(n,1); end
  if nargin == 6, n=precond.nrows; x0=zeros(n,1); xe=x0; end
  if nargin == 5, n=precond.nrows; x0=zeros(n,1); xe=x0; end
  if nargin == 4, n=precond.nrows; x0=zeros(n,1); xe=x0; prtyp=0; end

  n=precond.nrows;
  info=0; 
  iter=0;
  xe=xe; % exact solution
%
% setup
%
  rnorm0=norm(rhs);
  if(rnorm0 == 0.0)
    rnorm0=1.0; 
  end
  rnormi0=norm(rhs,inf);
  anormi=normi_CSR(csra);
%
% initial residual
%
  x=x0;
  r=mv_CSR(csra,x);
  r=rhs-r;
  rnorm=norm(r);
  if(rnorm < tol*rnorm0) 
    return;
  end
%
  for iter=1:maxit 
     job=0;
     z=precond_apply_CS(precond,r,prtyp,job);
     rho=(r'*z);
%
     if(iter > 1)
        beta=rho/rho1;
        p=z+beta*p;
     else
        p=z;
     end
%
     q=mv_CSR(csra,p);
     alpha=rho/(p'*q);
     x=x+alpha*p;
     r=r-alpha*q;
     rnorm=norm(r);
     xnorm=norm(x);
     backerr=rnorm/(anormi*xnorm+rnormi0);
     fprintf('%s %d %s %1.8g %s %1.8g \n','it = ',iter,' rnorm/rnorm0 = ',rnorm/rnorm0, ' backerr', backerr)
     if (rnorm <= tol*rnorm0)
       break; 
     end 
     rho1=rho;
  end
  relres=rnorm/rnorm0;
%
% set info
%
  if(rnorm > tol*rnorm0) 
    info=1; 
  end    
%
