
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
%
      function [x,info,relres,its] = gmr(A,rhs,tol,maxit,restrt,M1,M2,x0)
%
      if nargin == 7, [n n]=size(A); x0=zeros(n,1); end
      if nargin == 6, [n n]=size(A); M2=eye(n); x0=zeros(n,1); end
      if nargin == 5, [n n]=size(A); M2=eye(n); M2=eye(n); x0=zeros(n,1); end
      if nargin == 4, [n n]=size(A); M2=eye(n); M2=eye(n); x0=zeros(n,1); restrt=10; end
      if nargin == 3, [n n]=size(A); M2=eye(n); M2=eye(n); x0=zeros(n,1); restrt=10; maxit=1000; end
      if nargin == 2, [n n]=size(A); M2=eye(n); M2=eye(n); x0=zeros(n,1); restrt=10; maxit=1000;  tol=1.0d-8; end

      info=0; 
      iter=0;
      [n n]=size(A);
%
% setup
%
      rnorm0=norm(rhs);
      if(rnorm0 == 0.0)
        rnorm0=1.0; 
      end
%
%     initial residual
%
      x=x0;
      r=rhs-A*x;
      rnorm=norm(r);
      if(rnorm < tol*rnorm0) 
        return;
      end


      its = 0;                                      
      info = 0;
%
      M=M1*M2;%M=eye(n);
      r = M \ ( rhs-A*x );
      rnorm0=norm(r);
      if(norm(r) <= rnorm0*tol) 
        its=0;
        return;
      end 
%
      [n,n] = size(A);                              
      m = restrt;
      V(1:n,1:m+1) = zeros(n,m+1);
      H(1:m+1,1:m) = zeros(m+1,m);
      cs(1:m) = zeros(m,1);
      sn(1:m) = zeros(m,1);
      e1    = zeros(n,1);
      e1(1) = 1.0;

     for iter = 1:maxit                 
      r = M \ ( rhs-A*x );
      V(:,1) = r / norm( r );
      s = norm( r )*e1;
      for i = 1:m,                    
	 w = M \ (A*V(:,i));
	 for k = 1:i,
	   H(k,i)= w'*V(:,k);
	   w = w - H(k,i)*V(:,k);
	 end
	 H(i+1,i) = norm( w );
	 V(:,i+1) = w / H(i+1,i);
	 for k = 1:i-1,               
            temp     =  cs(k)*H(k,i) + sn(k)*H(k+1,i);
            H(k+1,i) = -sn(k)*H(k,i) + cs(k)*H(k+1,i);
            H(k,i)   = temp;
	 end
	 [cs(i),sn(i)] = rotmat( H(i,i), H(i+1,i) ); 
         temp   = cs(i)*s(i);                        
         s(i+1) = -sn(i)*s(i);
	 s(i)   = temp;
         H(i,i) = cs(i)*H(i,i) + sn(i)*H(i+1,i);
         H(i+1,i) = 0.0;
	 rnorm=abs(s(i+1));
         relres=rnorm/rnorm0;
         fprintf('%s %d %s %1.8g \n','it = ',i,' rnorm/rnorm0 = ',rnorm/rnorm0)
	 if ( rnorm <= rnorm0*tol), 
	    y = H(1:i,1:i) \ s(1:i);                 
            x = x + V(:,1:i)*y;
            relres=rnorm/rnorm0;
            its=i+(iter-1)*restrt;
	    break;
	 end
      end     

      if (rnorm <= rnorm0*tol)
        its=i+(iter-1)*restrt;
        break
      end
      y = H(1:m,1:m) \ s(1:m);
      x = x + V(:,1:m)*y;                            
      r = M \ ( rhs-A*x );                              
      s(i+1) = norm(r);
      if (rnorm <= rnorm0*tol)
        relres=rnorm/rnorm0;
        its=i+(iter-1)*restrt;
        break
      end
      relres=rnorm/rnorm0;
   end



