%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
%
% Jacobi iterative method
%

  function [x,relres,iter,info] = jacobi(a,rhs,tol,maxit,x0)
%
  if nargin == 4, [n n]=size(a); x0=zeros(n,1); end
  if nargin == 3, [n n]=size(a); x0=zeros(n,1); maxit=1000; end
  if nargin == 2, [n n]=size(a); x0=zeros(n,1); maxit=1000; tol=1.0d-8; end
  info=0; 
  iter=0;
  x=x0;
  r=rhs-a*x;
  rnorm=norm(r);
  rnorm0=rnorm;
  scaleda=a;
  for i = 1:n
    scaleda(i,i) = 0;
    x(i) = 0;
  end
  for i = 1:n
    scaleda(i,1:n) = scaleda(i,1:n)/a(i,i);
  end
  for i = 1:n
    scaledrhs(i) = rhs(i)/a(i,i);
  end
  iter = 0;
  while (iter < maxit) 
    xold=x;
    for i = 1:n
      x(i) = scaledrhs(i)-scaleda(i,:)*xold;
    end
    iter = iter+1;
    rnorm=norm(rhs-a*x);
    fprintf('%s %d %s %1.8g \n','it = ',iter,' rnorm/rnorm0 = ',rnorm/rnorm0)
    if (rnorm <= tol*rnorm0)
      break; 
    end 
  end
  relres=rnorm/rnorm0;
 
