%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% Diagonal pivoting method with the Bunch-Kaufmann pivoting
%
%
  function [l,d,P]=bk(a)
  %
  % basic initializations   
  % 
  % find the maximum diagonal/off-diagonal magnitudes
  % 
  [n n]=size(a); 
  asave=a;
  l=zeros(n,n);
  d=zeros(n,n);
  P=eye(n);
  p=zeros(1,n);
  alpha=(1+sqrt(17))/8;
  npivots=0;  
  k=1;     
  while k <= n
    %
    % find lambda
    %
    lambda=0;
    for i=k+1:n
      if(abs(a(i,k)) >= lambda)
        s=i;
        lambda=abs(a(i,k));
      end
    end
    if(abs(a(k,k)) >= alpha * lambda)
      pos1=k;
      pivot_case=1;
    else 
      sigma=0; 
      for i=k+1:n
        if(abs(a(i,s)) >= sigma && i ~= s) 
          t=i;
          sigma=abs(a(i,s));
        end
      end
      if (abs(a(k,k))*sigma >= alpha *lambda*lambda)
        pos1=k;
        pivot_case=1;
      elseif (abs(a(t,t))  >= alpha * sigma)
        pos1=s;
        pivot_case=1;
      else
        pos2a=k;
        pos2b=s;
        pivot_case=2;
      end
    end
    %
    % actual updates
    %
    if(pivot_case == 1)
      if(pos1 ~= k)
        P=swapcols(k,pos1,P);
        a=swapcols(k,pos1,a);
        a=swaprows(k,pos1,a);
      end
      npivots=npivots+1;
      pivot_type(npivots)=1;
      pivot=a(k,k);
      a(k+1:n,k+1:n)=a(k+1:n,k+1:n)-a(k+1:n,k)*inv(pivot)*a(k,k+1:n);
      a(k,k)=pivot; 
      d(k,k)=pivot;
      a(k,k:n)=inv(pivot)*a(k,k:n);
      a(k:n,k)=a(k,k:n)';
      k=k+1;
    else
      if(pos2a == k && pos2b > k+1)
        P=swapcols(k+1,pos2b,P);
        a=swapcols(k+1,pos2b,a);
        a=swaprows(k+1,pos2b,a);
      elseif(pos2a == k+1) 
        P=swapcols(k,pos2b,P);
        a=swapcols(k,pos2b,a);
        a=swaprows(k,pos2b,a);
      else
        P=swapcols(k,pos2a,P);
        P=swapcols(k+1,pos2b,P);
        a=swapcols(k,pos2a,a);
        a=swaprows(k,pos2a,a);
        a=swapcols(k+1,pos2b,a);
        a=swaprows(k+1,pos2b,a);
      end
      pivot=a(k:k+1,k:k+1);
      npivots=npivots+1;
      pivot_type(npivots)=2;
      a(k+2:n,k+2:n)=a(k+2:n,k+2:n)-a(k+2:n,k:k+1)*inv(pivot)*a(k:k+1,k+2:n);
      a(k:n,k:k+1)=a(k:n,k:k+1)*inv(pivot);
      a(k:k+1,k+2:n)=a(k+2:n,k:k+1)';
      d(k:k+1,k:k+1)=pivot;
      k=k+2;
    end
  end % while
  l=tril(a);

