%
%
% Miroslav Tuma, Charles University: Codes for Sparse matrices in Numerical Methods
% RB reordering
%
%  function [p n1 info]=rb(a)
%
% purpose:
%   Red-Black reordering of a square (sparse) matrix  having a bipartite graph
%
%   starts in 1, uses bfs
%
  function [p]=rb(a)
  %
    [m n]=size(a);
    [ia,ja,aa]=a2csr(a);
    asave=csr2a(m,ia,ja,aa); % just for fun
%
    start=1;
    [m n]= size(a);
    [ncomp,icomp,nlcomp,nlevel,ilevel,jlevel]=bfs(m,ia,ja,start);
    p=zeros(n,1);
    red=0;
    for i=1:2:nlevel
      red=red+ilevel(i+1)-ilevel(i);
    end
    indr=1;
    indb=red+1;
    for i=1:2:nlevel
      for j=ilevel(i):ilevel(i+1)-1
         p(indr)=jlevel(j);
         indr=indr+1;
      end
    end
    for i=2:2:nlevel
      for j=ilevel(i):ilevel(i+1)-1
         p(indb)=jlevel(j);
         indb=indb+1;
      end
    end
    y=sort(p(1:indr-1));
    z=sort(p(indr:n));
    p=[y' z']';
  end
